﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.Windows.Threading;
using System.Diagnostics;

using MindFusion.RealTimeCharting.Wpf;

namespace APlot_RTChart
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        DispatcherTimer dispatcherTimer;
        Stopwatch Tmr = new Stopwatch();
        Random rand;
        int clusterSize = 1;
        int currCount = 0;
        double y1 = 0.6, y2 = 0.7, y3 = 0.8, y4 = 0.01;  // y# are filtered, randomly-varying values
        double PPS = 0;//points per second

        Series series1 = null;
        Series series2 = null;
        Series series3 = null;
        Series series4 = null;

        public Window1()
        {
            InitializeComponent();

            rtChart.FrameLeftThickness = 50;
            rtChart.FrameRightThickness = 100;
            rtChart.FrameTopThickness = 100;
            rtChart.FrameBottomThickness = 100;
            rtChart.FastScrollMode = true;

            rtChart.TooltipVisibility = Visibility.Visible;
            rtChart.ShowFallbackTooltip = true;            

            rtChart.ShowLegend = true;
            rtChart.LegendHeight = 70;
            
            rtChart.MajorGridSizeY = 50;
            rtChart.MajorGridSizeX = 2;      

            rtChart.XAxis.Title = "Cycle";
            rtChart.XAxis.Interval = 2;
            rtChart.XAxis.Length = 50;
            rtChart.XAxis.Origin = 0;
            rtChart.XAxis.LabelFormat = "0";
            rtChart.XAxis.PinLabels = false;

            Axis yAxis = new Axis();
            yAxis.Origin = 0.0;
            yAxis.Length = 1;
            yAxis.Interval = 0.1;
            yAxis.Title = "ΔRn";
            yAxis.TitleRotationAngle = -90.0;
            yAxis.TitleFontFamily = new FontFamily("Verdana");
            yAxis.LabelFontFamily = new FontFamily("Verdana");
            yAxis.TickLength = 5;
            yAxis.TitleOffset = 10;
            rtChart.YAxisCollection.Add(yAxis);

            yAxis = new Axis();
            yAxis.Origin = 0.0;
            yAxis.Length = 100;
            yAxis.Interval = 10;
            yAxis.Title = "Fluorescene";
            yAxis.TitleRotationAngle = -90.0;
            yAxis.TitleFontFamily = new FontFamily("Verdana");
            yAxis.LabelFontFamily = new FontFamily("Verdana");
            yAxis.TickLength = 5;
            yAxis.TitleOffset = 10;
            yAxis.LabelHorizontalAlignment = HorizontalAlignment.Right;
            rtChart.Y2AxisCollection.Add(yAxis);

            rtChart.XAxis.LabelFontFamily = new FontFamily("Verdana");
            rtChart.XAxis.TitleFontFamily = new FontFamily("Verdana");

            series1 = new Series(rtChart.YAxisCollection[0])
            {
                Stroke = new SolidColorBrush(Colors.Green),
                ScatterStroke = new SolidColorBrush(Colors.Green),
                Fill = new SolidColorBrush(Colors.Green),
                ScatterType = ScatterType.Diamond,
                Title = "Sample 1",
                TitleFontFamily = new FontFamily("Verdana"),
                TitleFontSize = 12

            };
            series2 = new Series(rtChart.YAxisCollection[0])
            {
                Stroke = new SolidColorBrush(Colors.Red),
                ScatterStroke = new SolidColorBrush(Colors.Red),
                Fill = new SolidColorBrush(Colors.Red),
                Title = "Sample 2",
                ScatterType = ScatterType.Diamond,
                TitleFontFamily = new FontFamily("Verdana"),
                TitleFontSize = 12
            };
           
            series3 = new Series(rtChart.Y2AxisCollection[0])
            {
                Stroke = new SolidColorBrush(Colors.Blue),
                ScatterStroke = new SolidColorBrush(Colors.Blue),
                Fill = new SolidColorBrush(Colors.Blue),
                Title = "Sample 3",
                ScatterType = ScatterType.Diamond,
                TitleFontFamily = new FontFamily("Verdana"),
                TitleFontSize = 12
            };

          
            series4 = new Series(rtChart.Y2AxisCollection[0])
            {
                Stroke = new SolidColorBrush(Colors.Orange),
                ScatterStroke = new SolidColorBrush(Colors.Orange),
                Fill = new SolidColorBrush(Colors.Orange),
                Title = "Sample 4",
                ScatterType = ScatterType.Diamond,
                TitleFontFamily = new FontFamily("Verdana"),
                TitleFontSize = 12
            };


            rtChart.SeriesCollection.Add(series1);
            rtChart.SeriesCollection.Add(series2);
            rtChart.SeriesCollection.Add(series3);
            rtChart.SeriesCollection.Add(series4);

            rtChart.TooltipAxis = rtChart.YAxisCollection[0];

            rand = new Random((int)DateTime.Now.Ticks);
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            //DispatcherTimer setup
            dispatcherTimer = new System.Windows.Threading.DispatcherTimer(DispatcherPriority.Send);
            dispatcherTimer.Tick += new EventHandler(dispatcherTimer_Tick);
            dispatcherTimer.Interval = TimeSpan.FromMilliseconds(240);
            dispatcherTimer.Start(); 
        }

        private void dispatcherTimer_Tick(object sender, EventArgs e)
        {
            rtChart.FastScrollMode = true;
            Point[] points1 = new Point[clusterSize];
            Point[] points2 = new Point[clusterSize];
            Point[] points3 = new Point[clusterSize];
            Point[] points4 = new Point[clusterSize];

            double minNewX = currCount - 20;

            for (int i = 0; i < clusterSize; i++)
            {
                y1 = 0.2 + rand.NextDouble() * 0.09;
                y2 = 0.3 + rand.NextDouble() * 0.07;
                y3 = 60 + 2 * rand.NextDouble();
                y4 = 65 + 1 * rand.NextDouble();

                points1[i] = new Point(currCount + i, y1);
                points2[i] = new Point(currCount + i, y2);
                points3[i] = new Point(currCount + i, y3);
                points4[i] = new Point(currCount + i, y4);
            }

            currCount += clusterSize;
            PPS = 1000d * currCount / Tmr.ElapsedMilliseconds;
           
            series1.Data.AddRange(points1);
            series2.Data.AddRange(points2);
            series3.Data.AddRange(points3);
            series4.Data.AddRange(points4);

            rtChart.Commit(minNewX);

            DisposeOldData(series1);
            DisposeOldData(series2);
            DisposeOldData(series3);
            DisposeOldData(series4);
        }

        private void DisposeOldData(Series series)
        {
            if (series.Data.Count > 100 &&
                series.Data[99].X < rtChart.XAxis.Origin)
                series.Data.RemoveRange(0, 100);
        }
    }
}
